{% extends 'base.html' %} {% block title %}
<title>Text message page</title>
{% endblock %} {% block content %}
<div class="d-flex flex-column mt-5">
  <h1 class="align-self-center">{{ text_msg }}</h1>
</div>
{% endblock %}
