$primary: #001f4e;
$danger: #760000;
$success: #027400;

$spacer: 1rem;
$spacers: (
  0: 0,
  1: $spacer * 0.25,
  2: $spacer * 0.5,
  3: $spacer,
  4: $spacer * 1.5,
  5: $spacer * 3,
  10: $spacer * 6,
  15: $spacer * 9,
  20: $spacer * 12,
);

@import "../node_modules/bootstrap/scss/bootstrap";
@import "stepper";
