{% extends 'base.html' %} {% block title %}
<title>Unauthorized</title>
{% endblock %} {% block content %}
<div class="d-flex flex-column mt-20">
  <h1 class="align-self-center">You are not authorized to use aup manager.</h1>
  <div class="align-self-center">
    <p class="fs-4">{{ login }}</p>
  </div>
  <div class="d-grid gap-2 col-1 mx-auto">
    <a
      class="btn btn-primary btn-lg align-middle rounded-pill"
      href="{{ url_for('logout') }}"
      role="button"
    >
      LOGOUT
    </a>
  </div>
</div>

{% endblock %}
